#!/usr/bin/env bash

set -u

ROOTDIR=${ROOTDIR:-`bzr root`}
if [ ! -d "$ROOTDIR"  ]; then
    echo "ROOTDIR '$ROOTDIR' doesn't exist" >&2
    exit 1
fi

DATABASES="
distinct
"

function setup_database() {
    local TESTDIR=$1
    local PGBINDIR=

    echo "## Starting postgres in $TESTDIR ##"
    mkdir -p "$TESTDIR/data"
    chmod 700 "$TESTDIR/data"

    export PGHOST="$TESTDIR"
    export PGDATA="$TESTDIR/data"
    for pgver in 9.1 8.4; do
        if [ -d /usr/lib/postgresql/$pgver ]; then
            PGBINDIR=/usr/lib/postgresql/$pgver/bin
            break
        fi
    done
    if [ -z "$PGBINDIR" ]; then
        echo "Cannot find valid parent for PGBINDIR"
        exit 1
    fi
    $PGBINDIR/initdb -E UNICODE -D $PGDATA
    # set up the database options file
    if [ ! -e $PGDATA/postgresql.conf ]; then
        echo "PostgreSQL data directory apparently didn't init"
    else
    (
        cat <<EOF
search_path='\$user,public,ts2'
log_statement='all'
log_line_prefix='[%m] %q%u@%d %c '
fsync = off
EOF
    ) > $PGDATA/postgresql.conf
    fi
    $PGBINDIR/initdb -A trust &>/dev/null
    $PGBINDIR/pg_ctl start -w -D $TESTDIR/data -l $TESTDIR/postgres.log -o "-F -k $TESTDIR -h ''"
    for db in $DATABASES; do
        $PGBINDIR/createdb --encoding UNICODE "$db" &>/dev/null
        $PGBINDIR/createlang plpgsql "$db"
    done
    $PGBINDIR/createuser --superuser --createdb "postgres" &>/dev/null
    # create the additional users we need via a psql script
    $PGBINDIR/psql -U postgres template1 <<EOF
CREATE ROLE client INHERIT;

CREATE USER client IN ROLE client;
EOF
    echo "To set your environment so psql will connect to this DB instance type:"
    echo "    export PGHOST=$TESTDIR"
    echo "## Done. ##"
    echo -n host=$TESTDIR dbname=distinct > $ROOTDIR/tmp/pg.dsn
}

setup_database $ROOTDIR/tmp/db1
