"""
A rate limiter, heavily based on https://chris6f.com/rate-limiting-with-redis
"""

import time
import math


class RateLimiter(object):

    def __init__(self, span, bucket_size, expires=None, tick=time.time):
        self.bucket_size = bucket_size
        self.buckets = int(math.floor(float(span) / bucket_size)) + 2
        self.span = span
        self.expires = span if expires is None else expires
        self.tick = tick

    def add(self, client, key, count=1):
        p = client.pipeline()
        bucket = self.bucket()
        key = "rl:%s:%s" % (self.span, key)
        p.hincrby(key, bucket, count)
        p.expire(key, self.expires)
        p.hdel(key, (bucket + 1) % self.buckets)
        p.hdel(key, (bucket + 2) % self.buckets)
        p.execute()

    def count(self, client, key, interval):
        p = client.pipeline()
        bucket = self.bucket()

        key = "rl:%s:%s" % (self.span, key)

        # Asking for an interval smaller than the bucket size rounds it up to
        # one bucket. We do this to avoid getting false negatives when
        # accidentally asking for too small an interval.
        count = max(int(math.floor(float(interval) / self.bucket_size)), 1)
        for i in xrange(0, count):
            p.hget(key, (bucket - i) % self.buckets)

        results = p.execute()
        total = 0
        for value in results:
            if value is not None:
                total += int(value)
        return total

    def bucket(self):
        return int((self.tick() / self.bucket_size) % self.buckets)
